@@ -33,7 +33,6 @@
 
 #include "seahorse-common.h"
 
-#include "libseahorse/seahorse-object-list.h"
 #include "libseahorse/seahorse-progress.h"
 #include "libseahorse/seahorse-util.h"
 
@@ -75,28 +74,22 @@ static SoupURI*
 get_http_server_uri (SeahorseHKPSource *self, const char *path)
 {
     g_autoptr(SoupURI) uri = NULL;
-    g_autofree gchar *server = NULL;
-    gchar *port;
+    g_autofree char *conf_uri = NULL;
 
-    g_object_get (self, "key-server", &server, NULL);
-    g_return_val_if_fail (server != NULL, NULL);
+    conf_uri = seahorse_place_get_uri (SEAHORSE_PLACE (self));
+    g_return_val_if_fail (conf_uri != NULL, NULL);
 
-    uri = soup_uri_new (NULL);
-    soup_uri_set_scheme (uri, SOUP_URI_SCHEME_HTTP);
-
-    /* If it already has a port then use that */
-    port = strchr (server, ':');
-    if (port) {
-        *port++ = '\0';
-        soup_uri_set_port (uri, atoi (port));
-    } else {
-        /* default HKP port */
-        soup_uri_set_port (uri, 11371);
+    if (strncasecmp (conf_uri, "hkp:", 4) == 0) {
+        g_autofree char *t = g_strdup_printf ("http:%s", conf_uri + 4);
+        uri = soup_uri_new (t);
+    } else if (strncasecmp (conf_uri, "hkps:", 5) == 0) {
+        g_autofree char *t = g_strdup_printf ("https:%s", conf_uri + 5);
+        uri = soup_uri_new (t);
     }
 
-    soup_uri_set_host (uri, server);
     soup_uri_set_path (uri, path);
 
+    g_debug ("HTTP Server URI: %s", soup_uri_to_string(uri, FALSE));
     return g_steal_pointer (&uri);
 }
 
@@ -197,216 +190,196 @@ dehtmlize (gchar *line)
 }
 
 /**
- * parse_hkp_date:
- * @text: The date string to parse, YYYY-MM-DD
- *
- *
- *
- * Returns: 0 on error or the timestamp
- */
-static unsigned int
-parse_hkp_date (const gchar *text)
+* flags: combintation of [rei] representing the key's status
+*
+* Parses the flags from the HKP output
+*
+* returns 0 on error or a combination of seahorse flags based on input
+**/
+static guint
+parse_hkp_flags (char *flags)
 {
-    int year, month, day;
-    struct tm tmbuf;
-    time_t stamp;
+    char flag = 0;
 
-    if (strlen (text) != 10 || text[4] != '-' || text[7] != '-')
-        return 0;
+    g_return_val_if_fail (flags, 0);
 
-    /* YYYY-MM-DD */
-    sscanf (text, "%4d-%2d-%2d", &year, &month, &day);
-
-    /* some basic checks */
-    if (year < 1970 || month < 1 || month > 12 || day < 1 || day > 31)
-        return 0;
-
-    memset (&tmbuf, 0, sizeof tmbuf);
-    tmbuf.tm_mday = day;
-    tmbuf.tm_mon = month - 1;
-    tmbuf.tm_year = year - 1900;
-    tmbuf.tm_isdst = -1;
-
-    stamp = mktime (&tmbuf);
-    return stamp == (time_t)-1 ? 0 : stamp;
-}
-
-static const gchar *
-get_fingerprint_string (const gchar *line)
-{
-    const gchar *p;
-
-    p = line;
-    while (*p && g_ascii_isspace (*p))
-        p++;
-
-    if (g_ascii_strncasecmp (p, "fingerprint=", 12) == 0)
-        return p + 12;
-
-    return NULL;
+    for (char *f = flags; *f; f++){
+        switch (*f){
+            case 'r':
+                flag |= SEAHORSE_FLAG_REVOKED;
+                break;
+            case 'e':
+                flag |= SEAHORSE_FLAG_EXPIRED;
+                break;
+            case 'd':
+                flag |= SEAHORSE_FLAG_DISABLED;
+                break;
+        }
+    }
+    return flag;
 }
 
 /**
-* response: The HKP server response to parse
-*
-* Extracts the key data from the HKP server response
-*
-* Returns A GList of keys
-**/
-static GList*
-parse_hkp_index (const gchar *response)
+ * parse_hkp_index:
+ * response: The HKP server response to parse
+ *
+ * Extracts the key data from the HKP server response
+ *
+ * Returns: (transfer full): The parsed list of keys
+ */
+GList *
+seahorse_hkp_parse_lookup_response (const char *response)
 {
-    /* Luckily enough, both the HKP server and NAI HKP interface to their
-     * LDAP server are close enough in output so the same function can
-     * parse them both. */
-
-    /* pub  2048/<a href="/pks/lookup?op=get&search=0x3CB3B415">3CB3B415</a> 1998/04/03 David M. Shaw &lt;<a href="/pks/lookup?op=get&search=0x3CB3B415">dshaw@jabberwocky.com</a>&gt; */
-
+    /*
+     * Use The OpenPGP HTTP Keyserver Protocol (HKP) to search and get keys
+     * https://tools.ietf.org/html/draft-shaw-openpgp-hkp-00#section-5
+     */
     g_auto(GStrv) lines = NULL;
-    gchar **l;
-
     SeahorsePgpKey *key = NULL;
-    SeahorsePgpSubkey *subkey_with_id = NULL;
     GList *keys = NULL;
-    GList *subkeys = NULL;
-    GList *uids = NULL;
-    SeahorseFlags flags;
+    guint key_total = 0, key_count = 0;
 
     lines = g_strsplit (response, "\n", 0);
-    for (l = lines; *l; l++) {
-        gchar *line, *t;
+    for (char **l = lines; *l; l++) {
+        char *line = *l;
+        g_auto(GStrv) columns = NULL;
 
-        line = *l;
-        dehtmlize (line);
+        if (!*line) {
+          g_debug ("HKP Parser: skip empty line");
+          continue;
+        }
 
         g_debug ("%s", line);
 
-        /* Start a new key */
-        if (g_ascii_strncasecmp (line, "pub ", 4) == 0) {
-            g_auto(GStrv) v = NULL;
-            gchar *fingerprint, *fpr = NULL;
-            const gchar *algo;
-            gboolean has_uid = TRUE;
-            SeahorsePgpSubkey *subkey;
+        /* split the line using hkp delimiter */
+        columns = g_strsplit_set (line, ":", 7);
 
-            t = line + 4;
-            while (*t && g_ascii_isspace (*t))
-                t++;
+        /* info header */
+        /* info:<version>:<count> */
+        if (g_ascii_strncasecmp (columns[0], "info", 4) == 0) {
+            if (!columns[1] && !columns[2]){
+                g_debug("HKP Parse: Invalid info line: %s", line);
+            } else {
+                key_total = strtol(columns[2], NULL, 10);
+            }
 
-            v = g_strsplit_set (t, " ", 3);
-            if (!v[0] || !v[1] || !v[2]) {
+        /* start a new key */
+        /* pub:<keyid>:<algo>:<keylen>:<creationdate>:<expirationdate>:<flags> */
+        } else if (g_ascii_strncasecmp (columns[0], "pub", 3) == 0) {
+            const char *fpr;
+            g_autofree char *fingerprint = NULL;
+            const char *algo = NULL;
+            g_autoptr (SeahorsePgpSubkey) subkey = NULL;
+            long created = 0, expired = 0;
+            g_autoptr(GDateTime) created_date = NULL;
+            g_autoptr(GDateTime) expired_date = NULL;
+            SeahorseFlags flags;
+
+            key_count++;
+
+            if (!columns[0] || !columns[1] || !columns[2] || !columns[3] || !columns[4]) {
                 g_message ("Invalid key line from server: %s", line);
                 continue;
             }
 
-            flags = SEAHORSE_FLAG_EXPORTABLE;
-
             /* Cut the length and fingerprint */
-            fpr = strchr (v[0], '/');
-            if (fpr == NULL) {
+            fpr = columns[1];
+            if (fpr == NULL)
                 g_message ("couldn't find key fingerprint in line from server: %s", line);
-                fpr = "";
-            } else {
-                *(fpr++) = 0;
-            }
 
             /* Check out the key type */
-            switch (g_ascii_toupper (v[0][strlen (v[0]) - 1])) {
-            case 'D':
-                algo = "DSA";
-                break;
-            case 'R':
-                algo = "RSA";
-                break;
-            default:
-                algo = "";
-                break;
-            };
+            switch (strtol(columns[2], NULL, 10)) {
+                case 1:
+                case 2:
+                case 3:
+                     algo = "RSA";
+                    break;
+                case 17:
+                    algo = "DSA";
+                    break;
+                default:
+                   break;
+            }
+            g_debug ("Algo: %s", algo);
 
-            /* Format the date for our parse function */
-            g_strdelimit (v[1], "/", '-');
-
-            /* Cleanup the UID */
-            g_strstrip (v[2]);
-
-            if (g_ascii_strcasecmp (v[2], "*** KEY REVOKED ***") == 0) {
-                flags |= SEAHORSE_FLAG_REVOKED;
-                has_uid = FALSE;
+            /* set dates */
+            /* created */
+            if (!columns[4]) {
+                g_debug ("HKP Parse: No created date for key on line: %s", line);
+            } else {
+                created = strtol (columns[4], NULL, 10);
+                if (created > 0)
+                    created_date = g_date_time_new_from_unix_utc (created);
             }
 
-            if (key) {
-                seahorse_pgp_key_set_uids (SEAHORSE_PGP_KEY (key), uids);
-                g_list_free_full (uids, g_object_unref);
-                seahorse_pgp_key_set_subkeys (SEAHORSE_PGP_KEY (key), subkeys);
-                g_list_free_full (subkeys, g_object_unref);
-                seahorse_pgp_key_realize (SEAHORSE_PGP_KEY (key));
-                uids = subkeys = NULL;
-                subkey_with_id = NULL;
-                key = NULL;
+            /* expires (optional) */
+            if (columns[5]) {
+                expired = strtol (columns[5], NULL, 10);
+                if (expired > 0)
+                    expired_date = g_date_time_new_from_unix_utc (expired);
             }
 
+            /* set flags (optional) */
+            flags = SEAHORSE_FLAG_EXPORTABLE;
+            if (columns[6])
+                flags |= parse_hkp_flags (columns[6]);
+
+            /* create key */
+            g_debug("HKP Parse: found new key");
             key = seahorse_pgp_key_new ();
-            keys = g_list_prepend (keys, key);
             g_object_set (key, "object-flags", flags, NULL);
 
             /* Add all the info to the key */
             subkey = seahorse_pgp_subkey_new ();
             seahorse_pgp_subkey_set_keyid (subkey, fpr);
-            subkey_with_id = subkey;
 
             fingerprint = seahorse_pgp_subkey_calc_fingerprint (fpr);
             seahorse_pgp_subkey_set_fingerprint (subkey, fingerprint);
-            g_free (fingerprint);
 
             seahorse_pgp_subkey_set_flags (subkey, flags);
-            seahorse_pgp_subkey_set_created (subkey, parse_hkp_date (v[1]));
-            seahorse_pgp_subkey_set_length (subkey, strtol (v[0], NULL, 10));
-            seahorse_pgp_subkey_set_algorithm (subkey, algo);
-            subkeys = g_list_prepend (subkeys, subkey);
+            seahorse_pgp_subkey_set_created (subkey, created_date);
+            seahorse_pgp_subkey_set_expires (subkey, expired_date);
+            seahorse_pgp_subkey_set_length (subkey, strtol (columns[3], NULL, 10));
+            if (algo)
+                seahorse_pgp_subkey_set_algorithm (subkey, algo);
+            seahorse_pgp_key_add_subkey (key, subkey);
 
-            /* And the UID if one was found */
-            if (has_uid) {
-                SeahorsePgpUid *uid = seahorse_pgp_uid_new (key, v[2]);
-                uids = g_list_prepend (uids, uid);
-            }
+            /* Now add it to the list */
+            keys = g_list_prepend (keys, key);
 
         /* A UID for the key */
-        } else if (key && g_ascii_strncasecmp (line, "    ", 4) == 0) {
-            SeahorsePgpUid *uid;
+        } else if (g_ascii_strncasecmp (columns[0], "uid", 3) == 0) {
+            g_autoptr (SeahorsePgpUid) uid = NULL;
+            g_autofree char *uid_string = NULL;
 
-            g_strstrip (line);
-            uid = seahorse_pgp_uid_new (key, line);
-            uids = g_list_prepend (uids, uid);
-
-        /* Signatures */
-        } else if (key && g_ascii_strncasecmp (line, "sig ", 4) == 0) {
-            /* TODO: Implement signatures */
-
-        } else if (key && subkey_with_id) {
-            const char *fingerprint_str;
-            g_autofree gchar *pretty_fingerprint = NULL;
-
-            fingerprint_str = get_fingerprint_string (line);
-            if (fingerprint_str == NULL)
+            if (!key) {
+                g_debug("HKP Parse: Warning: seen uid line before keyline, skipping");
                 continue;
+            }
 
-            pretty_fingerprint = seahorse_pgp_subkey_calc_fingerprint (fingerprint_str);
+            g_debug("HKP Parse: handle uid");
 
-            /* FIXME: we don't check that the fingerprint actually matches
-             * the key's ID.  We also don't validate the fingerprint at
-             * all; the keyserver may have returned some garbage and we
-             * don't notice. */
-            if (pretty_fingerprint[0] != 0)
-                seahorse_pgp_subkey_set_fingerprint (subkey_with_id, pretty_fingerprint);
+            if (!columns[0] || !columns[1] || !columns[2]) {
+                g_message ("HKP Parse: Invalid uid line from server: %s", line);
+                continue;
+            }
+
+            uid_string = g_uri_unescape_string (columns[1], NULL);
+            g_debug("HKP Parse: decoded uid string: %s", uid_string);
+
+            uid = seahorse_pgp_uid_new (key, uid_string);
+            seahorse_pgp_key_add_uid (key, uid);
         }
     }
 
-    if (key) {
-        seahorse_pgp_key_set_uids (SEAHORSE_PGP_KEY (key), g_list_reverse (uids));
-        g_list_free_full (uids, g_object_unref);
-        seahorse_pgp_key_set_subkeys (SEAHORSE_PGP_KEY (key), g_list_reverse (subkeys));
-        g_list_free_full (subkeys, g_object_unref);
-        seahorse_pgp_key_realize (SEAHORSE_PGP_KEY (key));
+    /* Make sure the keys are realized */
+    for (GList *k = keys; k; k = g_list_next (k))
+        seahorse_pgp_key_realize (SEAHORSE_PGP_KEY (k->data));
+
+    if (key_total != 0 && key_total != key_count) {
+        g_message ("HKP Parse; Warning: Issue during HKP parsing, only %d keys were parsed out of %d", key_count, key_total);
+    } else {
+        g_debug ("HKP Parse: %d keys parsed successfully", key_count);
     }
 
     return keys;
@@ -421,20 +394,19 @@ parse_hkp_index (const gchar *response)
 *
 * Returns NULL if there was no error. The error message else
 **/
-static gchar*
-get_send_result (const gchar *response)
+static char*
+get_send_result (const char *response)
 {
-    gchar **lines, **l;
-    gchar *t;
-    gchar *last = NULL;
+    g_auto(GStrv) lines = NULL;
+    char *last = NULL;
     gboolean is_error = FALSE;
 
     if (!*response)
         return g_strdup ("");
 
     lines = g_strsplit (response, "\n", 0);
-
-    for (l = lines; *l; l++) {
+    for (char **l = lines; *l; l++) {
+        g_autofree char *t = NULL;
 
         dehtmlize (*l);
         g_strstrip (*l);
@@ -442,24 +414,17 @@ get_send_result (const gchar *response)
         if (!(*l)[0])
             continue;
 
-        t = g_ascii_strdown (*l, -1);
-
         /* Look for the word 'error' */
+        t = g_ascii_strdown (*l, -1);
         if (strstr (t, "error"))
             is_error = TRUE;
 
-        g_free (t);
-
         if ((*l)[0])
             last = *l;
     }
 
     /* Use last line as the message */
-    last = is_error ? g_strdup (last) : NULL;
-
-    g_strfreev (lines);
-
-    return last;
+    return is_error ? g_strdup (last) : NULL;
 }
 
 /**
@@ -474,7 +439,7 @@ get_send_result (const gchar *response)
  * Returns: FALSE if no key is contained, TRUE else
  */
 static gboolean
-detect_key (const gchar *text, gint len, const gchar **start, const gchar **end)
+detect_key (const char *text, int len, const char **start, const char **end)
 {
     const gchar *t;
 
@@ -501,8 +466,8 @@ hkp_message_propagate_error (SeahorseHKPSource *self,
                              SoupMessage *message,
                              GError **error)
 {
-    gchar *server;
-    g_autofree gchar *text = NULL;
+    g_autofree char *uri = NULL;
+    g_autofree char *text = NULL;
 
     if (!SOUP_MESSAGE_IS_ERROR (message))
         return FALSE;
@@ -513,13 +478,17 @@ hkp_message_propagate_error (SeahorseHKPSource *self,
         return TRUE;
     }
 
-    g_object_get (self, "key-server", &server, NULL);
+    uri = seahorse_place_get_uri (SEAHORSE_PLACE (self));
 
     /* Make the body lower case, and no tags */
     text = g_strndup (message->response_body->data, message->response_body->length);
     if (text != NULL) {
+        char *text_lower;
+
         dehtmlize (text);
-        seahorse_util_string_lower (text);
+        text_lower = g_ascii_strdown (text, -1);
+        g_free (text);
+        text = text_lower;
     }
 
     if (text && strstr (text, "no keys"))
@@ -527,11 +496,12 @@ hkp_message_propagate_error (SeahorseHKPSource *self,
 
     if (text && strstr (text, "too many")) {
         g_set_error (error, HKP_ERROR_DOMAIN, 0,
-                     _("Search was not specific enough. Server “%s” found too many keys."), server);
+                     _("Search was not specific enough. Server “%s” found too many keys."),
+                     uri);
     } else {
         g_set_error (error, HKP_ERROR_DOMAIN, message->status_code,
                      _("Couldn’t communicate with server “%s”: %s"),
-                     server, message->reason_phrase);
+                     uri, message->reason_phrase);
     }
 
     return TRUE;
@@ -550,7 +520,7 @@ typedef struct {
     GCancellable *cancellable;
     gulong cancelled_sig;
     SoupSession *session;
-    gint requests;
+    int requests;
     GcrSimpleCollection *results;
 } source_search_closure;
 
@@ -583,7 +553,7 @@ on_search_message_complete (SoupSession *session,
         return;
     }
 
-    keys = parse_hkp_index (message->response_body->data);
+    keys = seahorse_hkp_parse_lookup_response (message->response_body->data);
     for (l = keys; l; l = g_list_next (l)) {
         g_object_set (l->data, "place", closure->source, NULL);
         gcr_simple_collection_add (closure->results, l->data);
@@ -594,31 +564,29 @@ on_search_message_complete (SoupSession *session,
 }
 
 static gboolean
-is_hex_keyid (const gchar *match)
+is_hex_keyid (const char *match)
 {
-    const gchar *text = match;
+    size_t match_len;
 
-    if (strlen (match) != 8)
+    /* See HKP draft, section 3.1.1.1 */
+    match_len = strlen (match);
+    if (match_len != 8 || match_len != 16 || match_len != 32 || match_len != 40)
         return FALSE;
 
-    while (*text != '\0') {
-        if (!((*text >= 0x30 && *text <= 0x39) ||
-              (*text >= 0x41 && *text <= 0x46) ||
-              (*text >= 0x61 && *text <= 0x66)))
+    for (size_t i = 0; i < match_len; i++)
+        if (!g_ascii_isxdigit (match[i]))
             return FALSE;
-        text++;
-    }
 
     return TRUE;
 }
 
 static void
 seahorse_hkp_source_search_async (SeahorseServerSource *source,
-                                  const gchar *match,
-                                  GcrSimpleCollection *results,
-                                  GCancellable *cancellable,
-                                  GAsyncReadyCallback callback,
-                                  gpointer user_data)
+                                  const char           *match,
+                                  GcrSimpleCollection  *results,
+                                  GCancellable         *cancellable,
+                                  GAsyncReadyCallback   callback,
+                                  void                 *user_data)
 {
     SeahorseHKPSource *self = SEAHORSE_HKP_SOURCE (source);
     source_search_closure *closure;
@@ -626,7 +594,7 @@ seahorse_hkp_source_search_async (SeahorseServerSource *source,
     g_autoptr(GHashTable) form = NULL;
     SoupMessage *message;
     g_autoptr(SoupURI) uri = NULL;
-    gchar hexfpr[11];
+    g_autofree char *uri_str = NULL;
 
     task = g_task_new (source, cancellable, callback, user_data);
     closure = g_new0 (source_search_closure, 1);
@@ -641,10 +609,12 @@ seahorse_hkp_source_search_async (SeahorseServerSource *source,
 
     form = g_hash_table_new (g_str_hash, g_str_equal);
     g_hash_table_insert (form, "op", "index");
+    g_hash_table_insert (form, "options", "mr");
 
     if (is_hex_keyid (match)) {
-        strncpy (hexfpr, "0x", 3);
-        strncpy (hexfpr + 2, match, 9);
+        g_autofree char *hexfpr = NULL;
+
+        hexfpr = g_strdup_printf ("0x%s", match);
         g_hash_table_insert (form, "search", hexfpr);
     } else {
         g_hash_table_insert (form, "search", (char *)match);
@@ -657,6 +627,9 @@ seahorse_hkp_source_search_async (SeahorseServerSource *source,
 
     seahorse_progress_prep_and_begin (cancellable, message, NULL);
 
+    uri_str = soup_uri_to_string (uri, TRUE);
+    g_debug ("Sending HKP search query to '%s'", uri_str);
+
     soup_session_queue_message (closure->session, g_steal_pointer (&message),
                                 on_search_message_complete,
                                 g_steal_pointer (&task));
@@ -684,7 +657,7 @@ typedef struct {
     GCancellable *cancellable;
     gulong cancelled_sig;
     SoupSession *session;
-    gint requests;
+    int requests;
 } source_import_closure;
 
 static void
@@ -831,7 +804,7 @@ typedef struct {
     GCancellable *cancellable;
     gulong cancelled_sig;
     SoupSession *session;
-    gint requests;
+    int requests;
 } ExportClosure;
 
 static void
@@ -902,8 +875,6 @@ seahorse_hkp_source_export_async (SeahorseServerSource *source,
     g_autoptr(GTask) task = NULL;
     SoupURI *uri;
     g_autoptr(GHashTable) form = NULL;
-    gchar hexfpr[11];
-    gint i;
 
     task = g_task_new (self, cancellable, callback, user_data);
     closure = g_new0 (ExportClosure, 1);
@@ -921,23 +892,22 @@ seahorse_hkp_source_export_async (SeahorseServerSource *source,
     uri = get_http_server_uri (self, "/pks/lookup");
     g_return_if_fail (uri);
 
-    /* prepend the hex prefix (0x) to make keyservers happy */
-    strncpy (hexfpr, "0x", 3);
-
     form = g_hash_table_new (g_str_hash, g_str_equal);
-    for (i = 0; keyids[i] != NULL; i++) {
-        const gchar *fpr = keyids[i];
-        guint len;
+    for (int i = 0; keyids[i] != NULL; i++) {
+        const char *fpr = keyids[i];
+        size_t len;
+        g_autofree char *hexfpr = NULL;
         SoupMessage *message;
 
         g_hash_table_remove_all (form);
 
-        /* Get the key id and limit it to 8 characters */
+        /* Get the key id and limit it to 16 characters */
         len = strlen (fpr);
-        if (len > 8)
-            fpr += (len - 8);
+        if (len > 16)
+            fpr += (len - 16);
 
-        strncpy (hexfpr + 2, fpr, 9);
+        /* prepend the hex prefix (0x) to make keyservers happy */
+        hexfpr = g_strdup_printf ("0x%s", fpr);
 
         /* The get key URI */
         g_hash_table_insert (form, "op", "get");
@@ -992,9 +962,8 @@ seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass)
     server_class->export_finish = seahorse_hkp_source_export_finish;
     server_class->import_async = seahorse_hkp_source_import_async;
     server_class->import_finish = seahorse_hkp_source_import_finish;
-
-    seahorse_servers_register_type ("hkp", _("HTTP Key Server"), seahorse_hkp_is_valid_uri);
 }
+
 /**
  * seahorse_hkp_source_new:
  * @uri: The server to connect to
@@ -1004,13 +973,11 @@ seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass)
  * Returns: A new HKP Key Source
  */
 SeahorseHKPSource*
-seahorse_hkp_source_new (const gchar *uri, const gchar *host)
+seahorse_hkp_source_new (const char *uri)
 {
     g_return_val_if_fail (seahorse_hkp_is_valid_uri (uri), NULL);
-    g_return_val_if_fail (host && *host, NULL);
 
-    return g_object_new (SEAHORSE_TYPE_HKP_SOURCE, "uri", uri,
-                         "key-server", host, NULL);
+    return g_object_new (SEAHORSE_TYPE_HKP_SOURCE, "uri", uri, NULL);
 }
 
 /**
@@ -1029,9 +996,12 @@ seahorse_hkp_is_valid_uri (const gchar *uri)
 
     /* Replace 'hkp' with 'http' at the beginning of the URI */
     if (strncasecmp (uri, "hkp:", 4) == 0) {
-        g_autofree gchar *t = g_strdup_printf ("http:%s", uri + 4);
+        g_autofree char *t = g_strdup_printf ("http:%s", uri + 4);
         soup = soup_uri_new (t);
     /* Not 'hkp', but maybe 'http' */
+    } else if (strncasecmp (uri, "hkps:", 5) == 0) {
+        g_autofree char *t = g_strdup_printf ("https:%s", uri + 5);
+        soup = soup_uri_new (t);
     } else {
         soup = soup_uri_new (uri);
     }
